import numpy as np

def _cosine_window(size):
    cos_window = np.hanning(int(size[0]))[:, np.newaxis].dot(np.hanning(int(size[1]))[np.newaxis, :])
    cos_window = cos_window.astype(np.float32)
    cos_window /= np.sum(cos_window)
    return cos_window

def method_CAS(layer_out, channel_num, select_weight=1.0):
    window_inf = 0.8
    channel_cpu = layer_out[0].cpu().detach().numpy()
    channel_cpuoi = channel_cpu.copy()
    window = _cosine_window((channel_cpu[0].shape[0],channel_cpu[0].shape[1]))
    window_mean = np.mean(np.mean(window))
    for idx, cc in enumerate(channel_cpuoi):
        channel_cpuoi[idx] = (channel_cpuoi[idx] *window * window_inf + (1-window_inf) * window_mean*channel_cpuoi[idx])
    channel_sum_map = np.sum(channel_cpuoi, axis=0)
    summap_mean = np.mean(np.mean(channel_sum_map/channel_num))
    channel_selected = []
    for c in range(channel_cpu.shape[0]):
        tmp = channel_cpuoi[c].copy()
        if np.mean(np.mean(tmp)) > summap_mean * select_weight:
            channel_selected.append(c)
    return channel_selected

def torch2numpy(layer_out):
    return layer_out[0].cpu().detach().numpy()

def get_mask(size, mask_type='center'):
    if mask_type=='center':
        window = _cosine_window((size[0], size[1]))
    elif mask_type=='edge':
        window = _cosine_window((size[0], size[1]))
        window = window.max() - window
        window -= window.min()
        window /= window.sum()
    else:
        print('ERROR TYPE')
    return window

def compute_cosin_mask(X_i, mask, mask_inf=0.8):
    mask_mean  = np.mean(np.mean(mask))
    X_masked = (X_i * mask * mask_inf + (1 - mask_inf) * mask_mean * X_i)
    return X_masked

def target_aware(X, beta_c=1, beta_e=1, lam=0.01, mu=0.2):
    # init
    target_aware_score = []
    mask_size = (X.shape[1], X.shape[2])
    mask_c = get_mask(mask_size, 'center')
    mask_e = get_mask(mask_size, 'edge')
    channel_num = int(X.shape[0])

    X_muti_mask_c = X.copy()
    for idx, _ in enumerate(X_muti_mask_c):
        X_muti_mask_c[idx] = compute_cosin_mask(X_muti_mask_c[idx], mask_c, mask_inf=0.8)

    X_muti_mask_c_sum = np.sum(X_muti_mask_c, axis=0)
    X_muti_mask_c_mean = np.mean(np.mean(X_muti_mask_c_sum / channel_num))

    X_muti_mask_e = X.copy()
    for idx, _ in enumerate(X_muti_mask_e):
        X_muti_mask_e[idx] = compute_cosin_mask(X_muti_mask_e[idx], mask_e, mask_inf=0.8)

    X_muti_mask_e_sum = np.sum(X_muti_mask_e, axis=0)
    X_muti_mask_e_mean = np.mean(np.mean(X_muti_mask_e_sum / channel_num))

    for idx in range(channel_num):
        masked_c =   np.mean(np.mean(X_muti_mask_c[idx]))
        masked_e = np.mean(np.mean(X_muti_mask_e[idx]))
        CDAF_c = (masked_c - X_muti_mask_c_mean)
        CDAF_e = (masked_e - X_muti_mask_e_mean)
        panalty = lam  *(np.mean(np.mean((X_muti_mask_c[idx] - X_muti_mask_e[idx])**2)))
        TA = min(beta_c * CDAF_c , beta_e * CDAF_e) - panalty
        target_aware_score.append(TA)
    return  target_aware_score

def method_EAS(X):
    X = torch2numpy(X)
    sum_t = 0
    channel_selection=[]
    channel_score = target_aware(X)
    # return  channel_score
    clu = np.zeros((len(channel_score), 1))
    for i in range(len(channel_score)):
        clu[i] = i
    channel_score_sorted = np.column_stack((clu, channel_score))
    channel_score_sorted = channel_score_sorted[np.lexsort(channel_score_sorted.T)]
    for i in range(len(channel_score)):
        sum_plus = channel_score_sorted[i][1]
        if sum_plus>=sum_t:
            sum_t = sum_plus
            channel_selection.append(int(channel_score_sorted[i][0]))
    return channel_selection


def cxy_wh_2_rect1(pos, sz):
    """ convert (cx, cy, w, h) to (x1, y1, w, h), 1-index
    """
    return np.array([pos[0]-sz[0]/2+1, pos[1]-sz[1]/2+1, sz[0], sz[1]])

def rank_choice(layer_out, channel_num, select_num=80):
    gap = []
    for cc in layer_out:
        gap.append(np.sum(np.sum(cc)))
    clu = np.zeros((channel_num, 1))
    for i in range(channel_num):
        clu[i] = i
    rk = np.column_stack((clu, gap))
    rk = rk[np.lexsort(rk.T)]
    channel_selection=[]
    for i in range(select_num):
        channel_selection.append(int(rk[channel_num-1  - i][0]))
    return channel_selection

def channel_select_win_rank(layer_out, select_num=80):
    channel_cpu = layer_out[0].cpu().detach().numpy()
    channel_selected = rank_choice(channel_cpu, channel_cpu.shape[0],select_num=select_num)
    return channel_selected